/*
 * YAFFS: Yet another FFS. A NAND-flash specific file system. 
 * yaffs_mtdif.c  NAND mtd wrapper functions.
 *
 * Copyright (C) 2002 Aleph One Ltd.
 *   for Toby Churchill Ltd and Brightstar Engineering
 *
 * Created by Charles Manning <charles@aleph1.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
#ifdef CONFIG_YAFFS_MTD_ENABLED
 
#include "yportenv.h"

#include "yaffs_mtdif.h"

#include "linux/mtd/mtd.h"
#include "linux/types.h"


int nandmtd_WriteChunkToNAND(yaffs_Device *dev,int chunkInNAND,const __u8 *data, yaffs_Spare *spare)
{
	struct mtd_info *mtd = (struct mtd_info *)(dev->genericDevice);
	size_t dummy;
	
	loff_t addr = ((loff_t)chunkInNAND) * YAFFS_BYTES_PER_CHUNK;
	
	
	__u8 *spareAsBytes = (__u8 *)spare;
	
	if(data)
		mtd->write(mtd,addr,YAFFS_BYTES_PER_CHUNK,&dummy,data);
	if(spare)
		mtd->write_oob(mtd,addr,YAFFS_BYTES_PER_SPARE,&dummy,spareAsBytes);

	return YAFFS_OK;
}


int nandmtd_ReadChunkFromNAND(yaffs_Device *dev,int chunkInNAND, __u8 *data, yaffs_Spare *spare)
{
	struct mtd_info *mtd = (struct mtd_info *)(dev->genericDevice);
	size_t dummy;
	
	loff_t addr = ((loff_t)chunkInNAND) * YAFFS_BYTES_PER_CHUNK;
	
	__u8 *spareAsBytes = (__u8 *)spare;
	
	if(data)
		mtd->read(mtd,addr,YAFFS_BYTES_PER_CHUNK,&dummy,data);
	if(spare)
		mtd->read_oob(mtd,addr,YAFFS_BYTES_PER_SPARE,&dummy,spareAsBytes);

	return YAFFS_OK;
}


static void nandmtd_EraseCallback(struct erase_info *ei)
{
	yaffs_Device *dev = (yaffs_Device *)ei->priv;	
	up(&dev->sem);
}


int nandmtd_EraseBlockInNAND(yaffs_Device *dev, int blockNumber)
{
	
	struct mtd_info *mtd = (struct mtd_info *)(dev->genericDevice);
	__u32 addr = ((loff_t) blockNumber) * YAFFS_BYTES_PER_BLOCK;
	struct erase_info ei;
	
	ei.mtd = mtd;
	ei.addr = addr;
	ei.len = YAFFS_BYTES_PER_BLOCK;
	ei.time = 1000;
	ei.retries = 2;
	ei.callback = nandmtd_EraseCallback;
	ei.priv = (u_long)dev;
	
	// Todo finish off the ei if required
	
	sema_init(&dev->sem,0);

	mtd->erase(mtd,&ei);
	
	
	down(&dev->sem); // Wait for the erasure to complete
			
	//Todo, check result of erasure
	
	return YAFFS_OK;
}

int nandmtd_InitialiseNAND(yaffs_Device *dev)
{
	return YAFFS_OK;
}

#endif // CONFIG_YAFFS_MTD_ENABLED

